//AuthorSharpPlus Software Corp.
//Date2004-10-22
//ActionList for Winform core

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;
using SharpPlus.Action.Design;
//using OpenLicense;

namespace SharpPlus.Action
{
	/// <summary>
	/// Action is the generic class for action objects that implement user commands across a range of different controls.
	/// It can be added to action lists when there is no predefined action class that implements the desired response to 
	/// user commands. Unlike predefined action classes, which have built-in methods that respond when users click the client
	/// controls, Action has no built-in response to user commands. Instead, when using Action, you can provide the response 
	/// when the action "fires" by writing an ExecuteAction event handler, and configure the properties of Action to reflect
	/// current conditions by writing an UpdateAction event handler.																																																																																																																																																																													For predefined actions that augment the behavior of TAction, see the action classes in the StdActns, DBActns, and ExtActns units.
	/// </summary>
	[DesignTimeVisible(false), ToolboxItem(false)]
	public class Action:Component
	{
		/// <summary>
		/// 
		/// </summary>
		public Action():base()
		{
		}

		#region Property
		private ActionList _ActionList;
		/// <summary>
		/// Indicates the action list that contains the action.
		/// ActionList indicates which action list contains the action.An action list provides a visual interface for working
		/// with actions, which are then available to be associated with client controls in the application.
		/// </summary>
		[Browsable(false),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public ActionList ActionList
		{
			get {return _ActionList;}
			set {_ActionList=value;}
		}

		private string _Caption="";
		/// <summary>
		/// Represents the caption of client controls and menu items.
		/// Caption holds the string that is the caption for the action when it is set. The value of Caption is propagated to 
		/// client controls and menu items. 
		/// </summary>
		[Category("Common")]
		public string Caption
		{
			get {return _Caption;}
			set {_Caption=value;
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetCaption(target, value);
				}
			}
		}

		private bool _Enabled=true;
		/// <summary>
		/// Indicates whether client controls and menu items are enabled.
		/// Enabled specifies the enabled state for the action. The value of Enabled is propagated to client controls and menu items.
		/// </summary>
		[DefaultValue(true), Category("Common")]
		public bool Enabled
		{
			get {return _Enabled;}
			set {
				_Enabled=value;
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetEnabled(target, value);
				}
			}
		}

		private bool _Visible=true;
		/// <summary>
		/// Specifies the Visible property for client controls and menu items.
		/// Visible specifies the visible state for the action. This value is propagated to client controls and menu items.
		/// </summary>
		[DefaultValue(true), Category("Common")]
		public bool Visible
		{
			get {  return _Visible; }
			set { _Visible = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetVisible(target, value);
				}
			}    
		}

		private string _Tooltip="";
		/// <summary>
		/// Indicates the Help tooltip for client controls and menu items.
		/// Tooltip holds the string that is the hint for that action when it is set. This value is propagated to client controls and menu items.
		/// </summary>
		[DefaultValue("")]
		public string Tooltip
		{
			get {  return _Tooltip; }
			set { _Tooltip = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetTooltip(target, value);
				}
			}    
		}

		private bool _Checked;
		/// <summary>
		///	Indicates whether client controls and menu items appear checked.
		///	Checked specifies the checked state for the action. The value of Checked is propagated to client controls and menu items.
		///	</summary>
        [DefaultValue(false)]
		public bool Checked
		{
			get {  return _Checked; }
			set { _Checked = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetChecked(target, value);
				}
			}    
		}

		private object _Tag;
		/// <summary>
		/// Tag has no predefined meaning. The Tag property is provided for the convenience of developers. It can be used for an value
		/// such as a component reference.
		/// </summary>
		[Browsable(false), DefaultValue(null)]
		public object Tag
		{
			get {  return _Tag; }
			set { _Tag = value; }    
		}

		private string _Category="";
		/// <summary>
		/// Specifies the category under which the action appears in its action list.
		/// Category is the group, or category, to which the action belongs. An action list displays its actions grouped by category. 
		/// </summary>
        [Editor(typeof(CategoryEditor), typeof(System.Drawing.Design.UITypeEditor)), DefaultValue(""), Category("Common")]
		public string Category
		{
			get {return _Category;}
			set {_Category=value;}
		}

		private int _ImageIndex=-1;
		/// <summary>
		/// Indicates the ImageIndex property for client controls and menu items.
		/// ImageIndex is an index into the ImageList property of the action list that contains the action. It identifies an image that
		/// is associated with controls and menu items that use this action.
		/// </summary>
		[TypeConverter(typeof(IndexConverter)), Editor(typeof(ImageIndexEditor), typeof(System.Drawing.Design.UITypeEditor)), DefaultValue(-1), Category("Common")]
		public int ImageIndex
		{
			get {  return _ImageIndex; }
			set { _ImageIndex = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetImageIndex(target, value);
				}
			}    
		}

		private Shortcut _ShortCut;
		/// <summary>
		/// Specifies the ShortCut property for client menu items.
		/// ShortCut holds the value that is the shortcut for the action when it is set. This value is propagated to client menu items.
		/// </summary>
		[DefaultValue(Shortcut.None)]
		public Shortcut ShortCut
		{
			get {  return _ShortCut; }
			set { _ShortCut = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetShortcut(target, value);
				}
			}    
		}

		private bool _ShowShortcut=true;
		/// <summary>
		/// ShowShortcut
		/// </summary>
		[DefaultValue(true)]
		public bool ShowShortcut
		{
			get {  return _ShowShortcut; }
			set { _ShowShortcut = value; 
				foreach (object target in Targets)
				{
					ActionExtender ext=ExtenderManager.GetActionExtender(target);
					if (ext!=null)
						ext.SetShowShortcut(target, value);
				}
			}    
		}

		private bool _ShowToolBarButtonCaption=false;
		/// <summary>
		/// ShowToolBarButtonCaption
		/// </summary>
		[DefaultValue(false)]
		public bool ShowToolBarButtonCaption
		{
			get {  return _ShowToolBarButtonCaption; }
			set { _ShowToolBarButtonCaption = value; }    
		}

		#endregion

		#region Method

		//Bind and UnBind to target
		private ArrayList Targets=new ArrayList(5);
		internal void BindTarget(object target)
		{
			ActionExtender ext=ExtenderManager.GetActionExtender(target);
			if (ext!=null)
				ext.BindTarget(target, this);

			Targets.Add(target);
		}
		internal void UnbindTarget(object target)
		{
			ActionExtender ext=ExtenderManager.GetActionExtender(target);
			if (ext!=null)
				ext.UnBindTarget(target, this);

			Targets.Remove(target);
		}

		internal void EndInit()
		{
			foreach (object target in Targets)
			{
				ActionExtender ext=ExtenderManager.GetActionExtender(target);
				if (ext!=null)
					ext.EndInit(target, this);
			}
		}

		/// <summary>
		/// Introduces an interface for a method of notifying a client when the action updates itself.
		/// UpdateTarget does nothing in Action class. UpdateTarget was introduced in Action class so that descendants
		/// can override it to correspondingly update a target when the action updates.
		/// For examples, see the EditCopyAction class overrides the method to disable the client control when there is no text
		/// selected in active edit control.
		/// </summary>
		/// <returns></returns>
		public virtual bool UpdateTarget()
		{
			return _Enabled;
		}

		/// <summary>
		/// Responds when a client control "fires"
		/// Execute is called automatically when a client control "fires"(for example, when the user clicks a button or selects a menu item).
		/// ExecuteTarget does nothing in Action. ExecuteTarget was introduced in Action class so that descendants can override it to initiate
		/// the action on the target. For example, an edit action that performs copying might copy the contents of an edit control to the clipboard.
		/// </summary>
		protected virtual void ExecuteTarget()
		{
		}

		internal void DoExecute(object sender, System.EventArgs e)
		{
			if (this.ExecuteAction!=null)
			{
				this.ExecuteAction(sender, e);
			}
			else
				ExecuteTarget();
		}

		/// <summary>
		/// Execute calls the ExecuteAction event handler, if one is assigned, with the action as the Sender. Execute returns true if an event handler is called, false otherwise.
		/// </summary>
		/// <returns>bool</returns>
		public bool Execute()
		{
            return Execute(this);
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Sender"></param>
        /// <returns></returns>
        public bool Execute(Object Sender)
        {
            if (!UpdateTarget())
                return false;

            if (this.ExecuteAction != null)
            {
                this.ExecuteAction(Sender, null);
            }
            else
                ExecuteTarget();
            return true;
        }

		internal void OnMeasureItem( object sender, MeasureItemEventArgs e )
		{
			MenuItem menuItem = ( MenuItem )sender;
			MenuPainter MenuPainter = new MenuPainter( menuItem, e.Graphics, this );

			e.ItemHeight = MenuPainter.CalcHeight();
			e.ItemWidth  = MenuPainter.CalcWidth();
		}

		internal void OnDrawItem( object sender, DrawItemEventArgs e )
		{
			MenuItem menuItem = ( MenuItem )sender;
			MenuPainter MenuPainter = new MenuPainter( menuItem, e.Graphics, this );
			bool menuSelected = ( e.State & DrawItemState.Selected ) == DrawItemState.Selected;

			MenuPainter.DrawBackground( e.Bounds, menuSelected );

			if( MenuPainter.IsSeperator() )
				MenuPainter.DrawSeperator( e.Bounds );
			else
			{
				int imageIndex = this.ImageIndex;
				MenuPainter.DrawMenu( e.Bounds, menuSelected, imageIndex );
			}
		}

		internal void DoUpdate(object sender, System.EventArgs e)
		{
			if (this.UpdateAction!=null)
			{
				this.UpdateAction(sender, e);
			}
			else
				(sender as Action).Enabled=UpdateTarget();
		}

		#endregion

		#region Event
		/// <summary>
		/// Occurs when the client event that is linked to it fires.
		/// Write an ExecuteAction event handler when you want to respond when the user triggers
		/// the client object's default event (typically a Click event). For most target clients,
		/// ExecuteAction is triggered on a Click method, and is associated with the Click event.
		/// Warning:	If you assign an ExecuteAction event handler to a predefined action, the default behavior of that action will not occur.
		/// </summary>
		public event EventHandler ExecuteAction;
		/// <summary>
		/// Occurs when the application is idle or when the action list updates.
		/// Write an UpdateAction event handler to execute centralized code while an application is
		/// idle. For example, actions may want to update enabling and disabling, or checking and unchecking of client targets.
		/// </summary>
		public event EventHandler UpdateAction;
		#endregion

		#region MenuPainter Class

		private class MenuPainter
		{
			#region Private Attributes
			
			const int	SEPERATOR_HEIGHT	= 8;
			const int	BORDER_VERTICAL		= 2;
			const int	LEFT_MARGIN			= 4;
			const int	RIGHT_MARGIN		= 6;
			const int	SHORTCUT_MARGIN		= 20;
			const int	ARROW_MARGIN		= 12;
			const int	BULLET_DIAMETER		= 7;

			Action	extender;
			MenuItem		menuItem;
			Size			iconSize;
			Font			menuFont;
			Graphics		gfx;
			#endregion

			#region Constructors
			
			public MenuPainter( MenuItem item, Graphics graphics, Action ext )
			{
				menuItem	= item;
				extender	= ext;
				iconSize	= ( ext.ActionList.ImageList == null ) ? SystemInformation.SmallIconSize : ext.ActionList.ImageList.ImageSize;
				menuFont	= SystemInformation.MenuFont;
				gfx			= graphics;
			}
			#endregion

			#region Public Members
			
			public int CalcHeight()
			{
				if( IsSeperator() )
					return SEPERATOR_HEIGHT ;

				int itemHeight = ( menuFont.Height > iconSize.Height ) ? 
					menuFont.Height : iconSize.Height;

				return itemHeight + BORDER_VERTICAL;
			}
			
			public int CalcWidth()
			{
				StringFormat sf = new StringFormat() ;
				sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show ;
			
				int menuWidth = (int)Math.Ceiling( gfx.MeasureString( menuItem.Text, this.CurrentFont, 1000, sf).Width );
				int shortcutWidth = (int)Math.Ceiling( gfx.MeasureString( this.ShortcutText, this.CurrentFont, 1000, sf).Width );
				int arrowWidth = ( menuItem.IsParent ) ? ARROW_MARGIN : 0;

				if( IsTopLevel() )
					return menuWidth ;
				else
					return LEFT_MARGIN + iconSize.Width + RIGHT_MARGIN 
						+ menuWidth + SHORTCUT_MARGIN + shortcutWidth + arrowWidth;
			}
			
			public bool HasShortcut()
			{
				return ( menuItem.ShowShortcut && menuItem.Shortcut != Shortcut.None );
			}
			
			public bool IsSeperator()
			{
				return ( menuItem.Text == "-" );
			}
			
			public bool IsTopLevel()
			{
				return ( menuItem.Parent is MainMenu );
			}
			
			public string ShortcutText
			{
				get
				{
					if( menuItem.ShowShortcut && menuItem.Shortcut != Shortcut.None )
					{
						Keys keys = ( Keys )menuItem.Shortcut;
						return Convert.ToChar(Keys.Tab) + 
							System.ComponentModel.TypeDescriptor.GetConverter(keys.GetType()).ConvertToString(keys);
					}
					return null;
				}
			}

			public Font CurrentFont
			{
				get
				{
					if( menuItem.DefaultItem )
					{
						Font font;
						try
						{
							font = new Font( menuFont, menuFont.Style | System.Drawing.FontStyle.Bold );
						}
						catch
						{
							return menuFont;
						}
						return font;
					}
					return menuFont;
				}
				set { CurrentFont = value; }
			}

			public SolidBrush CurrentBrush( bool selected )
			{
				if( !menuItem.Enabled )
					return new SolidBrush( SystemColors.GrayText );

				SolidBrush menuBrush = null;
				if( selected )
					menuBrush = new SolidBrush( SystemColors.HighlightText );
				else
					menuBrush = new SolidBrush( SystemColors.MenuText );
				return menuBrush;
			}

			public void DrawBackground( Rectangle bounds, bool selected )
			{
				if( selected )
					gfx.FillRectangle( SystemBrushes.Highlight, bounds );
				else
					gfx.FillRectangle( SystemBrushes.Menu, bounds );
			}

			public void DrawMenu( Rectangle bounds, bool selected, int indexValue )
			{
				DrawMenuText( bounds, selected );
				
				if( menuItem.Checked )
					DrawCheckBox( bounds, selected );
				else
				{
					if( indexValue > -1 )
					{
						Image menuImage = extender.ActionList.ImageList.Images[ indexValue ];						
						DrawImage( menuImage, bounds );
					}
				}
			}
			
			public void DrawSeperator( Rectangle bounds )
			{
				Pen pen = new Pen( SystemColors.ControlDark );
				int yCenter	= bounds.Top  + (bounds.Height / 2);

				gfx.DrawLine(pen, bounds.Left, yCenter, (bounds.Left + bounds.Width), yCenter);
			}
			#endregion

			#region Private Members

			private void DrawMenuText( Rectangle bounds, bool selected )
			{
				SolidBrush menuBrush = CurrentBrush( selected );
				
				StringFormat sfMenu = new StringFormat();
				sfMenu.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show;

				int yPos = bounds.Top + (bounds.Height - this.CurrentFont.Height) / 2;

				gfx.DrawString( menuItem.Text, this.CurrentFont, menuBrush, 
					bounds.Left + LEFT_MARGIN + iconSize.Width + RIGHT_MARGIN, yPos, sfMenu );

				if( ! IsTopLevel() && HasShortcut() )
				{
					sfMenu.FormatFlags |= StringFormatFlags.DirectionRightToLeft;

					gfx.DrawString(this.ShortcutText, this.CurrentFont, menuBrush, 
						bounds.Width - ARROW_MARGIN, yPos, sfMenu);
				}
			}			
			
			private void DrawCheckBox( Rectangle bounds, bool selected )
			{
				Rectangle rectCheck = new Rectangle( bounds.Left, bounds.Top,
					SystemInformation.MenuCheckSize.Width,
					SystemInformation.MenuCheckSize.Height );

				rectCheck.X += ((LEFT_MARGIN + iconSize.Width + RIGHT_MARGIN) - rectCheck.Width) / 2;
				rectCheck.Y += (bounds.Height - rectCheck.Height) / 2;

				if( menuItem.RadioCheck )
					DrawBullet( rectCheck, selected );
				else
					DrawCheckMark( rectCheck, selected );
			}

			private void DrawBullet( Rectangle rect, bool selected )
			{
				SolidBrush menuBrush = CurrentBrush( selected );
				int x = rect.Left + ( rect.Width - BULLET_DIAMETER ) / 2;
				int y = rect.Top + ( rect.Height - BULLET_DIAMETER ) / 2;
				gfx.FillEllipse( menuBrush, x, y, BULLET_DIAMETER, BULLET_DIAMETER );
			}

			private void DrawCheckMark( Rectangle rect, bool selected )
			{
				SolidBrush brush = CurrentBrush( selected );
				Pen pen = new Pen( brush, 1 );

				int x = rect.Left + rect.Width  / 2;
				int y = rect.Top  + rect.Height / 2;
				Point[] points = new Point[] { new Point(x-4, y-1),
												 new Point(x-4, y+1),
												 new Point(x-2, y+3),
												 new Point(x+2, y-1),
												 new Point(x+2, y-3),
												 new Point(x-2, y+1),
												 new Point(x-4, y-1)
											 };
				gfx.FillPolygon( brush, points );
				gfx.DrawLines( pen, points );
			}
			
			private void DrawImage( Image menuImage, Rectangle bounds )
			{
				if( menuItem.Enabled )
					gfx.DrawImage( menuImage, bounds.Left + LEFT_MARGIN, 
						bounds.Top + ((bounds.Height - iconSize.Height) / 2), 
						iconSize.Width, iconSize.Height );	
				else
					ControlPaint.DrawImageDisabled(gfx, menuImage, 
						bounds.Left + LEFT_MARGIN, 
						bounds.Top + ((bounds.Height - iconSize.Height) / 2), 
						SystemColors.Menu );
			}
			#endregion
		}
		#endregion

	}

	/// <summary>
	/// 
	/// </summary>
	public class ActionCollection : CollectionBase
	{
		private ActionList _ActionList;

		internal ActionCollection(ActionList ActnList)
		{
			this._ActionList=ActnList;
		}

		/// <summary>
		/// 
		/// </summary>
		public Action this[int Index]
		{
			get
			{
				return (Action) List[Index];
			}
			set
			{
				List[Index]=value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="AAction"></param>
		/// <returns></returns>
		public int IndexOf(Action AAction)
		{
			return List.IndexOf(AAction);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Action"></param>
		/// <returns></returns>
		public bool Contains(Action Action)
		{
			return List.Contains(Action);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Action"></param>
		/// <returns></returns>
		public int Add(Action Action)
		{
			int i = List.Add(Action);
			Action.ActionList=this._ActionList;
			_ActionList.HookIdle();

			return i;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Action"></param>
		public void Remove(Action Action)
		{
			if ((Action.Site != null) && (Action.Site.Container != null))
			{
				Action.Site.Container.Remove(Action);
			}

			Action.ActionList=null;
			List.Remove(Action);
			Action.Dispose();
		}
	}

	/// <summary>
	/// Use Action lists to centralize the response to user commands (actions). Action list components maintain 
	/// a list of actions that are available to the client controls in an application. Add action list components 
	/// to your form from the component toolbox. Double-click the action list to display the action list designer,
	/// from which you can add, delete, and rearrange actions.
	/// </summary>
	/// 
	[ProvideProperty( "Action", typeof(Component)) ]
	[Designer(typeof(ActionDesigner))]
	//[LicenseProvider( typeof( OpenLicense.OpenLicenseProvider ))]
	public class ActionList:Component,IExtenderProvider, ISupportInitialize
	{
		/// <summary>
		/// 
		/// </summary>
		public ActionList()
		{
			try
			{
				//validate License 
				//OpenLicense.OpenLicenseProvider.ResetCashe( typeof( ActionList ) );
				//LicenseManager.Validate( typeof( ActionList ) );
				//OpenLicense.OpenLicenseProvider provider=new OpenLicense.OpenLicenseProvider();
				//License lic=provider.GetLicense(null,typeof(ActionList),null,true);

//				DateTime ExpireDate=new DateTime(2006,7,15);
//				if (DateTime.Now>ExpireDate)
//				{
//					throw new Exception("SharpPlus ActionList Trial License already expired, please contact service@sharpplus.com ");
//				}
				_Actions=new ActionCollection(this);

				//DesignMode always false in constructor
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		private bool IdleHooked=false;
		internal void HookIdle()
		{
			if (DesignMode || IdleHooked)
				return;
			Application.Idle +=new EventHandler(Application_Idle);
			IdleHooked=true;
		}

		private ActionCollection _Actions;

		/// <summary>
		/// Lists the actions maintained by the action list.
		/// Actions is an indexed property containing a list of the actions in the action list component.
		/// Index identifies the action by index, where 0 is the index of the first action, 1 is the index
		/// of the second action, and so on.
		/// </summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public ActionCollection Actions
		{
			get
			{
				return _Actions;
			}
		}

		/// <summary>
		/// Ensure actions is freed when actinlist is deleted
		/// </summary>
		/// <param name="disposing"></param>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (IdleHooked)
				  Application.Idle-=new EventHandler(Application_Idle);
				while (this._Actions.Count > 0)
				{
					int i=this._Actions.Count-1;
					Action Action = this._Actions[i];
					this._Actions.Remove(Action);
				}
				this._Actions = null;
			}
			base.Dispose(disposing);
		}

		#region IExtenderProvider Member

		/// <summary>
		/// 
		/// </summary>
		/// <param name="extendee"></param>
		/// <returns></returns>
		public bool CanExtend(object extendee)
		{
			return ExtenderManager.GetActionExtender(extendee)!=null;
		}

		private Hashtable _Maps=new Hashtable(50);
		/// <summary>
		/// 
		/// </summary>
		/// <param name="pComponent"></param>
		/// <param name="Action"></param>
		public void SetAction( Component pComponent, Action Action )
		{
			if( !_Maps.Contains(pComponent))
			{
				if (pComponent==null || Action==null)
					return;

				_Maps.Add(pComponent, Action);

				Action.BindTarget(pComponent);
			}
			else 
			{
				Action oldAction=(Action)_Maps[pComponent];

				if (Action==null)
				{
					_Maps.Remove(pComponent);
					oldAction.UnbindTarget(pComponent);
				}
				else
				{
					Action.BindTarget(pComponent);
					_Maps[pComponent ] = Action;
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="pComponent"></param>
		/// <returns></returns>
		[Category("ActionList")]
		[Editor(typeof(ActionEditor), typeof(System.Drawing.Design.UITypeEditor))]
		[DefaultValue(null)]
		public Action GetAction( Component pComponent )
		{
			if( _Maps.Contains( pComponent ))
			{
				Action Action=(Action) _Maps[ pComponent ];
				//Clear reference with action
				if (this._Actions.Contains(Action))
					return Action;
				else
					return null;
			}

			return null;
		}

		#endregion

		private ImageList _ImageList=null;
		/// <summary>
		/// Lists the images available for actions in the action list.
		/// An action is associated with an image from this image list by
		/// its ImageIndex property. This image can then appear on client
		/// controls such as menu items, toolbar buttons, and so on.
		/// </summary>
		[DefaultValue(null)]
		public System.Windows.Forms.ImageList ImageList
		{
			get {  return _ImageList; }
			set { _ImageList = value; }    
		}

		private void Application_Idle(object sender, EventArgs e)
		{
			if (this.DesignMode) 
				return;

			//Execute UpdateTarget
			foreach (Action Action in this.Actions)
			{
				Action.DoUpdate(Action, null);
			}
		}
		#region ISupportInitialize Member

		/// <summary>
		/// 
		/// </summary>
		public void BeginInit()
		{
			// do nothing
		}

		/// <summary>
		/// 
		/// </summary>
		public void EndInit()
		{
			//Support Toolbar binding
			if (this.DesignMode)
				return;
			foreach (Action Action in this.Actions)
			{
				Action.EndInit();
			}
		}

		internal void DoToolbarExecute(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
			Action AAction=this.GetAction(e.Button);
			if (AAction!=null)
				AAction.DoExecute(e.Button, null);
		}

		#endregion
	}

	/// <summary>
	/// ActionManager contains an array predefined action classes
	/// </summary>
	public class ActionManager
	{
		private static SortedList _ActionCategories=new SortedList(5);
		private static ActionManager _Manager;
		private ActionManager()
		{
			//Register Actions
			RegisterAction("Internet",typeof(BrowseUrlAction));
			RegisterAction("Edit", typeof(EditCopyAction));
			RegisterAction("Edit", typeof(EditCutAction));
			RegisterAction("Edit", typeof(EditPasteAction));
			RegisterAction("Edit", typeof(EditSelectAllAction));
			RegisterAction("Edit", typeof(EditUndoAction));
			RegisterAction("Edit", typeof(EditClearAction));
			RegisterAction("Window", typeof(WindowCloseAction));
			RegisterAction("Window", typeof(WindowArrangeAction));
			RegisterAction("Window", typeof(WindowCascadeAction));
			RegisterAction("Window", typeof(WindowMinimizeAllAction));
			RegisterAction("Window", typeof(WindowTileHorizontalAction));
			RegisterAction("Window", typeof(WindowTileVerticalAction));
			RegisterAction("ListBox", typeof(ListClearAction));
			RegisterAction("ListBox", typeof(ListDeleteSelectionAction));
			RegisterAction("ListBox", typeof(ListSelectAllAction));
			RegisterAction("ListBox", typeof(ListCopySelectionAction));
			RegisterAction("ListBox", typeof(ListMoveSelectionAction));
		}

		/// <summary>
		/// Registers action classes so that they appear in the list of predefined standard actions in the standard actions dialog.
		/// RegisterAction allows a class to appear in the action list editor or customize dialog as a selection when you choose New Standard Action. To install your own action classes so that they can be added to an action list, call RegisterActions to register them.
		/// </summary>
		/// <param name="Category"> The Category parameter specifies the value of the actions's Category property.</param>
		/// <param name="ActionType">The ActionType parameter is type of a custom action class.</param>
		public static void RegisterAction(string Category, Type ActionType)
		{
			SortedList actList=(SortedList)_ActionCategories[Category];
			if (actList==null)
			{
				actList=new SortedList(5);
				_ActionCategories.Add(Category, actList);
			}
			if (!actList.ContainsKey(ActionType.ToString()))
				actList.Add(ActionType.ToString(), ActionType);
		}

		internal static SortedList GetActionCategories()
		{
			if (_Manager==null)
				_Manager=new ActionManager();
			return _ActionCategories;
		}
	}
}
