//AuthodSharpPlus Software Corp.
//All standard actions
//Date:2004-12-4

using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Forms;

namespace SharpPlus.Action
{
	#region Internet Actions
	/// <summary>
	/// BrowseUrlAction is the standard action for launching the default browser to display a specified URL.
	/// Controls linked to this action cause the application to launch the default browser, as specified in the system registry,
	/// and display the Web page specified by the URL property. You should assign a value to the Url property before the user invokes this action.
	/// </summary>
	public class BrowseUrlAction:Action
	{
		/// <summary>
		/// 
		/// </summary>
		public BrowseUrlAction()
		{
			this.Category="Internet";
		}

		#region Property
		private string _Url="";
		/// <summary>
		///Specifies the Uniform Resource Locator (URL) on which this action performs its function.
		///Url is the Uniform Resource Locator (URL) with which this action works.This value is not
		///assigned automatically: it is the responsibility of the application to supply a value for
		///Url. If no value is supplied, BrowseUrlAction automatically disables itself. 
		/// </summary>
		[DefaultValue(""), Category("Internet")]
		public string Url
		{
			get {return _Url;}
			set 
			{
				_Url=value;
			}
		}
		#endregion

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			Process.Start(Url);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return this.Url.Trim()!=string.Empty;
		}

	}
	#endregion

	#region Edit Actions
	/// <summary>
	/// EditAction is the base class for action objects that perform standard edits on an edit control.
	/// EditAction is the common base class for actions that work with edit controls. Each descendant 
	/// class performs a specific type of edit on the target edit control. Do not use EditAction directly
	/// in an application. Instead, use a EditAction descendant that performs a specific function 
	/// (for example, Cut, Copy, Paste, Delete, Select All, or Undo).
	/// </summary>
	public class EditAction:Action
	{
		/// <summary>
		/// 
		/// </summary>
		public EditAction()
		{
			this.Category="Edit";
		}

		#region Property
		private TextBoxBase _TextBox;
		/// <summary>
		/// TextBox is the edit control on which the action performs its function. Set the value of Control
		/// to limit the action so that it only operates on a specific edit control. If Control is unassigned,
		/// the action dynamically locates the target edit control when it executes. 
		/// </summary>
		[DefaultValue(null),Category("Edit")]
		public System.Windows.Forms.TextBoxBase TextBox
		{
			get {  return _TextBox; }
			set { _TextBox = value; }    
		}
	
		#endregion

		internal bool HasSelectedText()
		{
			TextBoxBase txt=GetCurrTextBox();
			if (txt==null)
				return false;
			return txt.SelectedText!=string.Empty;
		}

		internal TextBoxBase GetCurrTextBox()
		{
			if (this.TextBox!=null)
				return this.TextBox;
			if (Form.ActiveForm==null)
				return null;
			Form AForm=Form.ActiveForm;
			if (AForm.IsMdiContainer && AForm.ActiveMdiChild!=null)
			{
				AForm=AForm.ActiveMdiChild;
			}

			if (AForm.ActiveControl==null)
				return null;
			if (!(AForm.ActiveControl is TextBoxBase))
				return null;
			return ((TextBoxBase)AForm.ActiveControl);
		}
	}

	/// <summary>
	/// EditCopyAction copies the selected text in a target edit control to the Windows clipboard.
	/// Add EditCopyAction to an action list to allow users to copy selected text to the clipboard using menu
	/// items or toolbar buttons. Controls such as menu items and tool buttons linked to this action cause the
	/// application to copy the currently selected text in an active text control to the clipboard.
	/// EditCopyAction is designed to work with an edit control (TextBoxBase descendant) as a target. It automatically
	/// disables itself if the active control in a form is not an edit control or if that edit control does not have
	/// any text selected.
	/// </summary>
	public class EditCopyAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		public EditCopyAction():base()
		{
			this.ShortCut=Shortcut.CtrlC;
		}

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().Copy();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return HasSelectedText();
		}
	}

	/// <summary>
	/// EditCutAction cuts the selected text in a target edit control to the Windows clipboard.
	/// Add EditCutAction to an action list to allow users to cut selected text to the clipboard using menu items
	/// or toolbar buttons. Controls such as menu items and tool buttons linked to this action cause the application
	/// to cut the currently selected text in an active text control to the clipboard.
	/// EditCutAction is designed to work with an edit control (TextBoxBase descendant) as a target. It automatically
	/// disables itself if the active control in a form is not an edit control or if that edit control does not have 
	/// a selection to cut.
	/// </summary>
	public class EditCutAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		public EditCutAction():base()
		{
			this.ShortCut=Shortcut.CtrlX;
		}

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().Cut();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return HasSelectedText();
		}
	}

	/// <summary>
	/// EditClearAction deletes all the text in a target edit control.
	/// Add EditClearAction to an action list to let users delete selected text using menu items
	/// or toolbar buttons. Controls such as menu items and tool buttons linked to this action
	/// cause the application to delete all the text in an active text control.
	/// EditClearAction is designed to work with an edit control (TextBoxBase descendant) as a target.
	/// </summary>
	public class EditClearAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().Clear();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return GetCurrTextBox()!=null;
		}
	}

	/// <summary>
	/// EditPasteAction pastes text from the clipboard into the active edit control.
	/// Add EditPasteAction to an action list to enable users to paste text from the clipboard using menu items
	/// or toolbar buttons. Controls such as menu items and tool buttons linked to this action cause the application
	/// to paste text from the clipboard into an active text control.
	/// EditPasteAction is designed to work with an edit control (TextBoxBase descendant) as a target.
	/// It automatically disables itself if the clipboard does not contain text.
	/// </summary>
	public class EditPasteAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		public EditPasteAction():base()
		{
			this.ShortCut=Shortcut.CtrlV;
		}

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().Paste();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return GetCurrTextBox()!=null && Clipboard.GetDataObject().GetDataPresent(DataFormats.Text);
		}
	}

	/// <summary>
	/// EditSelectAllAction selects all text in an active text control.
	/// Add EditSelectAllAction to an action list to allow users to select all the text in a text control using menu items
	/// or toolbar buttons. Controls such as menu items and tool buttons linked to this action cause the application to
	/// select all text in an active text control.
	/// EditSelectAllAction is designed to work with an edit control (TextBoxBase descendant) as a target. 
	/// It automatically disables itself if the active control in a form is not an edit control or if the edit control does not have any text.
	/// </summary>
	public class EditSelectAllAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		public EditSelectAllAction():base()
		{
			this.ShortCut=Shortcut.CtrlA;
		}

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().SelectAll();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			if (GetCurrTextBox()==null)
				return false;
			return (GetCurrTextBox().Text!=string.Empty);
		}
	}

	/// <summary>
	/// EditUndoAction backs out the last change in an edit or memo control.
	/// Add EditUndoAction to an action list to allow users to back out edits using menu items
	/// or toolbar buttons. Controls such as menu items and tool buttons linked to this action
	/// cause the application to undo the last edit in an active text control.
	/// EditUndoAction is designed to work with an edit control (TextBoxBase descendant) as a target.
	/// It automatically disables itself if the active control in a form is not an edit or memo control that can back out the last edit.
	/// </summary>
	public class EditUndoAction:EditAction
	{
		/// <summary>
		/// 
		/// </summary>
		public EditUndoAction():base()
		{
			this.ShortCut=Shortcut.CtrlZ;
		}

		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrTextBox().Undo();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			if (GetCurrTextBox()==null)
				return false;
			return GetCurrTextBox().CanUndo;
		}
	}
	#endregion

	#region Window Actions
	/// <summary>
	/// WindowAction is the base class for action objects that work with forms especially MDI forms.
	/// WindowAction implements the common behavior for actions that work on MDI forms. Each descendant class
	/// performs a specific task, such as arranging, cascading, closing, tiling, or minimizing MDI child forms.
	/// Do not use WindowAction directly in an application. Instead, use a WindowAction descendant such as WindowCloseAction
	/// , WindowCascadeAction, WindowMinimizeAllAction, WindowArrangeAction, WindowTileHorizontalAction, or WindowTileVerticalAction. 
	/// </summary>
	public class WindowAction:Action
	{
		/// <summary>
		/// 
		/// </summary>
		public WindowAction():base()
		{
			this.Category="Window";
		}

		#region Property
		private Form _Form;
		/// <summary>
		/// Form is the current form on which the action performs its function.
		/// </summary>
		[DefaultValue(null), Category("Window")]
		public System.Windows.Forms.Form Form
		{
			get {  return _Form; }
			set { _Form = value; }    
		}
	
		#endregion

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		protected Form GetCurrWindow()
		{
			if (this.Form!=null)
				return this.Form;
			return Form.ActiveForm;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		protected bool HasMdiChild()
		{
			if (this.Form!=null)
				return Form.MdiChildren.Length>0;
			else
			{
				if (Form.ActiveForm==null)
					return false;
				return Form.ActiveForm.MdiChildren.Length>0;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return HasMdiChild();
		}

	}

	/// <summary>
	/// WindowCloseAction closes the active form.
	/// Add WindowCloseAction to an action list to allow users to close the active form.
	/// Controls such as menu items and tool buttons linked to this action cause the 
	/// application to call the Close method of the active form.
	/// WindowCloseAction is designed to work with an active form as a target.
	/// It automatically disables itself if the active form is null.
	/// </summary>
	public class WindowCloseAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			GetCurrWindow().Close();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return GetCurrWindow()!=null;
		}
	}

	/// <summary>
	/// WindowArrangeAction arranges the icons of minimized MDI child forms.
	/// Add WindowArrangeAction to an action list to allow users to arrange the icons that represent minimized MDI child forms.
	/// Controls such as menu items and tool buttons linked to this action cause the application to arrange the MDI child forms to the icons
	/// WindowArrangeAction is designed to work with an MDI parent form as a target.
	/// It automatically disables itself if the active form is not an MDI parent or if it has no MDI children.
	/// </summary>
	public class WindowArrangeAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			this.GetCurrWindow().LayoutMdi(MdiLayout.ArrangeIcons);
		}
	}

	/// <summary>
	/// WindowCascadeAction cascades the MDI child forms.
	/// Add WindowCascadeAction to an action list to allow users to arrange the MDI child forms in an overlapping cascade.
	/// WindowCascadeAction is designed to work with an MDI parent form as a target.
	/// It automatically disables itself if the active form is not an MDI parent or if it has no MDI children.
	/// </summary>
	public class WindowCascadeAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			this.GetCurrWindow().LayoutMdi(MdiLayout.Cascade);
		}
	}

	/// <summary>
	/// WindowMinimizeAll minimizes all MDI child forms.
	/// Add WindowMinimizeAll to an action list to allow users to minimize the MDI child forms in an MDI application.
	/// Controls such as menu items and tool buttons linked to this action cause the application to iterate through the MDI child forms,
	/// setting their WindowState to Minimized.
	/// WindowMinimizeAll is designed to work with an MDI parent form as a target.
	/// It automatically disables itself if the active form is not an MDI parent or if it has no MDI children.
	/// </summary>
	public class WindowMinimizeAllAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			foreach (Form child in GetCurrWindow().MdiChildren)
			{
				child.WindowState=FormWindowState.Minimized;
			}
		}
	}

	/// <summary>
	/// WindowTileHorizontalAction arranges MDI child forms so that they are all the same size, tiled horizontally.
	/// Add WindowTileHorizontalAction to an action list to allow users to tile the MDI child forms horizontally.
	/// Controls such as menu items and tool buttons linked to this action cause the MDI parent form to tile its children horizontally.
	/// WindowTileHorizontalAction is designed to work with an MDI parent form as a target.
	/// It automatically disables itself if the active form is not an MDI parent or if it has no MDI children.
	/// </summary>
	public class WindowTileHorizontalAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			this.GetCurrWindow().LayoutMdi(MdiLayout.TileHorizontal);
		}
	}

	/// <summary>
	/// WindowTileVerticalAction arranges MDI child forms so that they are all the same size, tiled vertically.
	/// Add WindowTileVerticalAction to an action list to allow users to tile the MDI child forms vertically.
	/// Controls such as menu items and tool buttons linked to this action cause the MDI parent form to tile its children vertically.
	/// WindowTileVerticalAction is designed to work with an MDI parent form as a target.
	/// It automatically disables itself if the active form is not an MDI parent or if it has no MDI children.
	/// </summary>
	public class WindowTileVerticalAction:WindowAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			this.GetCurrWindow().LayoutMdi(MdiLayout.TileVertical);
		}

	}

	#endregion

	#region Search Actions

	#endregion

	#region List Actions

    /// <summary>
    /// 
    /// </summary>
	public class ListAction:Action
	{
        /// <summary>
        /// 
        /// </summary>
		public ListAction():base()
		{
			this.Category="List";
		}

		private ListBox _ListBox;
        /// <summary>
        /// 
        /// </summary>
		[DefaultValue(null), Category("ListBox")]
		public System.Windows.Forms.ListBox SourceList
		{
			get {  return _ListBox; }
			set { _ListBox = value; }    
		}

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
		protected bool IsEnabled()
		{
           return SourceList!=null && /*SourceList.Focused &&*/ SourceList.Items.Count>0 && SourceList.Enabled;
		}
	}

    /// <summary>
    /// 
    /// </summary>
	public class ListClearAction:ListAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			SourceList.ClearSelected();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return IsEnabled() && SourceList.SelectedItems.Count>0;
		}
	}

    /// <summary>
    /// 
    /// </summary>
	public class ListDeleteSelectionAction:ListAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			for (int i=SourceList.SelectedItems.Count-1;i>=0;i--)
			{
				SourceList.Items.Remove(SourceList.SelectedItems[i]);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return IsEnabled() && SourceList.SelectedItems.Count>0;
		}
	}

    /// <summary>
    /// 
    /// </summary>
	public class ListSelectAllAction:ListAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			for (int i=0;i<SourceList.Items.Count;i++)
              SourceList.SetSelected(i,true);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return IsEnabled() && ((SourceList.SelectionMode==SelectionMode.MultiSimple) || (SourceList.SelectionMode==SelectionMode.MultiExtended));
		}
	}

    /// <summary>
    /// 
    /// </summary>
	public class ListCopySelectionAction:ListAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			for (int i=0;i<SourceList.SelectedItems.Count;i++)
			{
				DestList.Items.Add(SourceList.SelectedItems[i]);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return IsEnabled() && SourceList.SelectedItems.Count>0 && DestList!=null && DestList.Enabled;
		}

		ListBox _DestList;
        /// <summary>
        /// 
        /// </summary>
		[DefaultValue(null), Category("ListBox")]
		public System.Windows.Forms.ListBox DestList
		{
			get {  return _DestList; }
			set { _DestList = value; }    
		}
	}

    /// <summary>
    /// 
    /// </summary>
	public class ListMoveSelectionAction:ListAction
	{
		/// <summary>
		/// 
		/// </summary>
		protected override void ExecuteTarget()
		{
			for (int i=0;i<SourceList.SelectedItems.Count;i++)
			{
				DestList.Items.Add(SourceList.SelectedItems[i]);
			}

			for (int i=SourceList.SelectedItems.Count-1;i>=0;i--)
			{
				SourceList.Items.Remove(SourceList.SelectedItems[i]);
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override bool UpdateTarget()
		{
			return IsEnabled() && SourceList.SelectedItems.Count>0 && DestList!=null && DestList.Enabled;
		}

		/// <summary>
		/// 
		/// </summary>
		ListBox _DestList;
        /// <summary>
        /// 
        /// </summary>
		[DefaultValue(null), Category("ListBox")]
		public System.Windows.Forms.ListBox DestList
		{
			get {  return _DestList; }
			set { _DestList = value; }    
		}
	}
	#endregion
}
